/*
    Information about server communication. This sample webservice is provided by Wikitude and returns random dummy
    places near given location.
 */
var ServerInformation = {
   // POIDATA_SERVER: "http://pk-advent.indigital.gr/api/points-of-interest/",
//    POIDATA_SERVER: "https://example.wikitude.com/GetSamplePois/",
    POIDATA_SERVER_ARG_LAT: "lat",
    POIDATA_SERVER_ARG_LON: "lon",
    POIDATA_SERVER_ARG_NR_POIS: "nrPois"
};

/* Implementation of AR-Experience (aka "World"). */
var World = {

    /*
        User's latest known location, accessible via userLocation.latitude, userLocation.longitude,
         userLocation.altitude.
     */
    userLocation: null,

    /* You may request new data from server periodically, however: in this sample data is only requested once. */
    isRequestingData: false,

    /* True once data was fetched. */
    initiallyLoadedData: false,

    language: null,

    audio: null,

    filterData: "Σημεία Ενδιαφέροντος",

    /* Different POI-Marker assets. */
    markerDrawableIdle: null,
    markerLandMarkDrawableIdle: null,
    markerDrawableSelected: null,
    markerLandMarkDrawableSelected: null,
    markerDrawableDirectionIndicator: null,

    /* List of AR.GeoObjects that are currently shown in the scene / World. */
    markerList: [],

    /* the last selected marker. */
    currentMarker: null,

    locationUpdateCounter: 0,
    updatePlacemarkDistancesEveryXLocationUpdates: 10,

    loadLanguage: function loadLanguageFn(lang) {
        language = lang;
    },

    /* Called to inject new POI data. */
    loadPoisFromJsonData: function loadPoisFromJsonDataFn(poiData) { //I need to put poiData on function parameter to work as intended.

        document.getElementById("rangeTitle").innerHTML = (language == 'el') ? "Εμβέλεια" : "Range";
        document.getElementById("rangeButton").innerHTML = (language == 'el') ? "Εμβέλεια" : "Range";
        document.getElementById("listButton").innerHTML = (language == 'el') ? "Λίστα Σημείων" : "POI List";
        document.getElementById("closePanelRangeButton").innerHTML = (language == 'el') ? "Κλείσε" : "Close";
        document.getElementById("closePanelPoiDetailButton").innerHTML = (language == 'el') ? "Κλείσε" : "Close";
        document.getElementById("#poiDetailMoreButton").innerHTML = (language == 'el') ? "Περισσότερα" : "More";
        document.getElementById("rangeLabel").innerHTML = (language == 'el') ? 'Εμβέλεια: <a id="panelRangeValue"></a>' : 'Range: <a id="panelRangeValue"></a>';
        document.getElementById("visibleLabel").innerHTML = (language == 'el') ? 'Ορατά: <a id="panelRangePlaces"></a>' : 'Visible: <a id="panelRangePlaces"></a>';
        document.getElementById("detailTitle").innerHTML = (language == 'el') ? "Λεπτομέρειες" : "Details";

        /* Destroys all existing AR-Objects (markers & radar). */
        AR.context.destroyAll();

        /* Show radar. */
        PoiRadar.show();

        /* Empty list of visible markers. */
        World.markerList = [];

        /* Start loading marker assets. */
        World.markerDrawableIdle = new AR.ImageResource("assets/marker_idle_new.png", {
            onError: World.onError
        });
        World.markerLandMarkDrawableIdle = new AR.ImageResource("assets/marker_idle_new-landmark-2.png", {
                    onError: World.onError
                });
        World.markerDrawableSelected = new AR.ImageResource("assets/marker_selected_new.png", {
            onError: World.onError
        });
        World.markerLandMarkDrawableSelected = new AR.ImageResource("assets/marker_selected_new-landmark-2.png", {
                    onError: World.onError
                });
        World.markerDrawableDirectionIndicator = new AR.ImageResource("assets/indi.png", {
            onError: World.onError
        });

        /* Loop through POI-information and create an AR.GeoObject (=Marker) per POI. */
        for (var currentPlaceNr = 0; currentPlaceNr < poiData.length; currentPlaceNr++) {
            var singlePoi = {
                "id": poiData[currentPlaceNr].id,
                "latitude": parseFloat(poiData[currentPlaceNr].lat),
                "longitude": parseFloat(poiData[currentPlaceNr].lng),
                "altitude": parseFloat(poiData[currentPlaceNr].altitude),
                "category_name" : poiData[currentPlaceNr].category_name,
                "title": poiData[currentPlaceNr].title,
                "title_en": poiData[currentPlaceNr].title_en,
                "description": poiData[currentPlaceNr].description,
                "description_en": poiData[currentPlaceNr].description_en,
                "pic": poiData[currentPlaceNr].pic,
                "video": poiData[currentPlaceNr].video != null ? poiData[currentPlaceNr].video : "",
                "sketchfab": poiData[currentPlaceNr].sketchfab != null ? poiData[currentPlaceNr].sketchfab : "",
                "three_image": poiData[currentPlaceNr].three_image != null ? poiData[currentPlaceNr].three_image : "",
                "narration" : poiData[currentPlaceNr].narration != null ? poiData[currentPlaceNr].narration : ""
            };

            World.markerList.push(new Marker(singlePoi, language));
        }

        /* Updates distance information of all placemarks. */
        World.updateDistanceToUserValues();

        World.updateStatusMessage(currentPlaceNr + ' places loaded');
        World.updateRangeValues();

        /* Set distance slider to 100%. */
        document.getElementById("panelRangeSliderValue").innerHTML = 100;
    },

    /*
        Sets/updates distances of all makers so they are available way faster than calling (time-consuming)
        distanceToUser() method all the time.
     */
    updateDistanceToUserValues: function updateDistanceToUserValuesFn() {
        for (var i = 0; i < World.markerList.length; i++) {
            World.markerList[i].distanceToUser = World.markerList[i].markerObject.locations[0].distanceToUser();
        }
    },

    /* Updates status message shown in small "i"-button aligned bottom center. */
    updateStatusMessage: function updateStatusMessageFn(message, isWarning) {
//        document.getElementById("popupButtonImage").src = isWarning ? "assets/warning_icon.png" : "assets/info_icon.png";
        document.getElementById("popupButtonTooltip").innerHTML = message;
    },

    /*
        It may make sense to display POI details in your native style.
        In this sample a very simple native screen opens when user presses the 'More' button in HTML.
        This demoes the interaction between JavaScript and native code.
    */
    /* User clicked "More" button in POI-detail panel -> fire event to open native screen. */
    onPoiDetailMoreButtonClicked: function onPoiDetailMoreButtonClickedFn() {
        var currentMarker = World.currentMarker;
        var markerSelectedJSON = {
            action: "present_poi_details",
            id: currentMarker.poiData.id,
            title: currentMarker.poiData.title,
            title_en: currentMarker.poiData.title_en,
            description: currentMarker.poiData.description,
            description_en: currentMarker.poiData.description_en,
            pic: currentMarker.poiData.pic,
            video: currentMarker.poiData.video,
            sketchfab: currentMarker.poiData.sketchfab,
            three_image: currentMarker.poiData.three_image
        };
        World.stopNarration();

        /*
            The sendJSONObject method can be used to send data from javascript to the native code.
        */
        AR.platform.sendJSONObject(markerSelectedJSON);
    },

    /* Location updates, fired every time you call architectView.setLocation() in native environment. */
    locationChanged: function locationChangedFn(lat, lon, alt, acc) {

        /* Store user's current location in World.userLocation, so you always know where user is. */
        World.userLocation = {
            'latitude': lat,
            'longitude': lon,
            'altitude': alt,
            'accuracy': acc
        };


        /* Request data if not already present. */
        if (!World.initiallyLoadedData) {
            World.requestDataFromServer(lat, lon);
            World.initiallyLoadedData = true;
        } else if (World.locationUpdateCounter === 0) {
            /*
                Update placemark distance information frequently, you max also update distances only every 10m with
                some more effort.
             */
            World.updateDistanceToUserValues();
        }

        /* Helper used to update placemark information every now and then (e.g. every 10 location upadtes fired). */
        World.locationUpdateCounter =
            (++World.locationUpdateCounter % World.updatePlacemarkDistancesEveryXLocationUpdates);
    },

    /*
        POIs usually have a name and sometimes a quite long description.
        Depending on your content type you may e.g. display a marker with its name and cropped description but
        allow the user to get more information after selecting it.
    */

    showList: function showListFn() {
        World.closeAndReleasePanel();

        document.getElementById("closePanelPoiListButton").innerHTML = (language == 'el') ? "Κλείσε" : "Close";
        document.getElementById("listTitle").innerHTML = (language == 'el') ? "Λίστα σημείων" : "Pois List";
        document.getElementById("filterButtonPoi").innerHTML = (language == 'el') ? "Σημεία Ενδιαφ." : "Pois";
        document.getElementById("filterButtonLandmarks").innerHTML = (language == 'el') ? "Γεωγρ. Ορόσημα" : "Landmarks";

       World.loadListTable();

        document.getElementById("panelPoiList").style.visibility = "visible";
    },

    loadListTable: function loadListTableFn() {
        var table = document.getElementById("poiListTable");

        table.innerHTML = "";

        var tbody = document.createElement("tbody");
        tbody.setAttribute("id", "poiListTableBody");


        for (var i = 0; i < World.markerList.length; i++) {

            if (World.markerList[i].distanceToUser < World.getMaxDistance() && World.markerList[i].poiData.category_name == World.filterData) {
                var row = document.createElement("tr");
                var rowData = document.createElement("td");
                rowData.innerHTML = language == "el" ? World.markerList[i].poiData.title : World.markerList[i].poiData.title_en;
                rowData.setAttribute("id", "tableDataRow");
                row.setAttribute("onclick", "World.selectedPoi("+i+");");
                row.appendChild(rowData);
                tbody.appendChild(row);
            }

        }

        table.appendChild(tbody);
    },

    selectedPoi: function selectedPoiFn(index) {
        World.closeAndReleasePanel();
        var marker = World.markerList[index];
        Marker.prototype.setSelected(marker)
        World.currentMarker = marker;

    },

    /* Fired when user pressed maker in cam. */
    onMarkerSelected: function onMarkerSelectedFn(marker) {
        World.closeAndReleasePanel();

        World.currentMarker = marker;

        /*
            In this sample a POI detail panel appears when pressing a cam-marker (the blue box with title &
            description), compare index.html in the sample's directory.
        */
        /* Update panel values. */
        document.getElementById("poiDetailTitle").innerHTML = (language == 'el') ? marker.poiData.title : marker.poiData.title_en;
        document.getElementById("poiDetailDescription").innerHTML = (language == 'el') ? marker.poiData.description.replace(/^(.{65}[^\s]*).*/, "$1")+" ..." : marker.poiData.description_en.replace(/^(.{61}[^\s]*).*/, "$1")+" ...";
        World.audio = marker.poiData.narration != "" ? new Audio("https://new-advent.indigital.gr/wp-content/uploads/narrations/"+marker.poiData.narration) : null;
        (language == 'el') ? World.playNarration() : null; //this should be removed once the english narration is added
        document.getElementById("poiDetailAltitudeLabel").innerHTML = (language == 'el') ? "Υψόμετρο: " : "Altitude: ";
        document.getElementById("poiDetailDistanceTitle").innerHTML = (language == 'el') ? "Απόσταση: " : "Distance: ";

        /*
            It's ok for AR.Location subclass objects to return a distance of `undefined`. In case such a distance
            was calculated when all distances were queried in `updateDistanceToUserValues`, we recalculate this
            specific distance before we update the UI.
         */
        if (undefined === marker.distanceToUser) {
            marker.distanceToUser = marker.markerObject.locations[0].distanceToUser();
        }

        /*
            Distance and altitude are measured in meters by the SDK. You may convert them to miles / feet if
            required.
        */
        var distanceToUserValue = (marker.distanceToUser > 999) ?
            ((marker.distanceToUser / 1000).toFixed(2) + " km") :
            (Math.round(marker.distanceToUser) + " m");

        document.getElementById("poiDetailDistance").innerHTML = distanceToUserValue;

        document.getElementById("poiDetailAltitude").innerHTML = marker.poiData.altitude + " m";

        /* Show panel. */
        document.getElementById("panelPoiDetail").style.visibility = "visible";
    },

    decodeHtml: function decodeHtmlFn(html) {
//        var txt = document.createElement("textarea");
//        txt.innerHTML = html;
        if ((html !== null) || (html !== '')) {
            html = html.toString();
             return html.replace( /(<([^>]+)>)/ig, '');
        }
//
    },

    playNarration: function playNarrationFn() {
        if (World.audio != null) {
            World.audio.play();
        }
    },

    stopNarration: function stopNarrationFn() {

        if (World.audio != null) {
            World.audio.pause();
        }

    },

    closeAndReleasePanel: function closeAndReleasePanelFn() {
         World.closePanel();

            if (World.currentMarker != null) {
                /* Deselect AR-marker when user exits detail screen div. */
                World.currentMarker.setDeselected(World.currentMarker);
                World.currentMarker = null;
            }
    },

    poiFilter: function poiFilterFn() {
        World.filterData = "Σημεία Ενδιαφέροντος";
       World.loadListTable();
    },

    landmarksFilter: function landmarksFilterFn() {
        World.filterData = "Γεωγραφικά Ορόσημα";
       World.loadListTable();
    },

    closePanel: function closePanel() {
        /* Hide panels. */
        document.getElementById("panelPoiDetail").style.visibility = "hidden";
        document.getElementById("panelPoiList").style.visibility = "hidden";
        document.getElementById("panelRange").style.visibility = "hidden";

        World.stopNarration();

    },

    /* Screen was clicked but no geo-object was hit. */
    onScreenClick: function onScreenClickFn() {
        /* You may handle clicks on empty AR space too. */
        World.closeAndReleasePanel();
    },

    /* Returns distance in meters of placemark with maxdistance * 1.1. */
    getMaxDistance: function getMaxDistanceFn() {

        /* Sort places by distance so the first entry is the one with the maximum distance. */
        World.markerList.sort(World.sortByDistanceSortingDescending);

        /* Use distanceToUser to get max-distance. */
        var maxDistanceMeters = World.markerList[0].distanceToUser;

        /*
            Return maximum distance times some factor >1.0 so ther is some room left and small movements of user
            don't cause places far away to disappear.
         */
        return 20000;  //maxDistanceMeters * 0.8 //I need to change the constant to reduce or increase the distance
    },

    /* Updates values show in "range panel". */
    updateRangeValues: function updateRangeValuesFn() {

        /* Get current slider value (0..100);. */
        var slider_value = document.getElementById("panelRangeSlider").value;
        /* Max range relative to the maximum distance of all visible places. */
        var maxRangeMeters = Math.round(World.getMaxDistance() * (slider_value / 100));

        /* Range in meters including metric m/km. */
        var maxRangeValue = (maxRangeMeters > 999) ?
            ((maxRangeMeters / 1000).toFixed(2) + " km") :
            (Math.round(maxRangeMeters) + " m");

        /* Number of places within max-range. */
        var placesInRange = World.getNumberOfVisiblePlacesInRange(maxRangeMeters);
        var placesLang = language == "el" ? " Σημεία" : " Places";
        var placeLang = language == "el" ? " Σημείο" : " Place";

        /* Update UI labels accordingly. */
        document.getElementById("panelRangeValue").innerHTML = maxRangeValue;
        document.getElementById("panelRangePlaces").innerHTML = (placesInRange != 1) ?
            (placesInRange + placesLang) : (placesInRange + placeLang);
        document.getElementById("panelRangeSliderValue").innerHTML = slider_value;

        World.updateStatusMessage((placesInRange != 1) ?
            (placesInRange + " places loaded") : (placesInRange + " place loaded"));

        /* Update culling distance, so only places within given range are rendered. */
        AR.context.scene.cullingDistance = Math.max(maxRangeMeters, 1);

        /* Update radar's maxDistance so radius of radar is updated too. */
        PoiRadar.setMaxDistance(Math.max(maxRangeMeters, 1));
    },

    /* Returns number of places with same or lower distance than given range. */
    getNumberOfVisiblePlacesInRange: function getNumberOfVisiblePlacesInRangeFn(maxRangeMeters) {

        /* Sort markers by distance. */
        World.markerList.sort(World.sortByDistanceSorting);
        var withinRange = 0;
        /* Loop through list and stop once a placemark is out of range ( -> very basic implementation ). */
        for (var i = 0; i < World.markerList.length; i++) {
            if (World.markerList[i].distanceToUser < maxRangeMeters) {
                withinRange++;
            }
        }
        /* In case no placemark is out of range -> all are visible. */
       // return World.markerList.length;
       return withinRange;
    },

    handlePanelMovements: function handlePanelMovementsFn() {
        PoiRadar.updatePosition();
    },

    /* Display range slider. */
    showRange: function showRangeFn() {
        if (World.markerList.length > 0) {
            World.closePanel();

            /* Update labels on every range movement. */
            World.updateRangeValues();
            World.handlePanelMovements();

            /* Open panel. */
            document.getElementById("panelRange").style.visibility = "visible";
        } else {

            /* No places are visible, because the are not loaded yet. */
            World.updateStatusMessage('No places available yet', true);
        }
    },

    /*
        You may need to reload POI information because of user movements or manually for various reasons.
        In this example POIs are reloaded when user presses the refresh button.
        The button is defined in index.html and calls World.reloadPlaces() on click.
    */

    /* Reload places from content source. */
    reloadPlaces: function reloadPlacesFn() {
        if (World.markerList.length > 0) {
            World.closePanel();
        }
        if (!World.isRequestingData) {
            if (World.userLocation) {
                World.requestDataFromServer(World.userLocation.latitude, World.userLocation.longitude);
            } else {
                World.updateStatusMessage('Unknown user-location.', true);
            }
        } else {
            World.updateStatusMessage('Already requesing places...', true);
        }
    },

    /* Request POI data. */
    requestDataFromServer: function requestDataFromServerFn(lat, lon) {

        /* Set helper var to avoid requesting places while loading. */
        World.isRequestingData = true;
        World.updateStatusMessage('Requesting places from web-service');

        /* Server-url to JSON content provider. */
//        var serverUrl = ServerInformation.POIDATA_SERVER + "?" + ServerInformation.POIDATA_SERVER_ARG_LAT + "=" +
//            lat + "&" + ServerInformation.POIDATA_SERVER_ARG_LON + "=" +
//            lon + "&" + ServerInformation.POIDATA_SERVER_ARG_NR_POIS + "=20";

            var serverUrl = ServerInformation.POIDATA_SERVER;

        /* Use GET request to fetch the JSON data from the server */
        var xhr = new XMLHttpRequest();
        xhr.open('GET', serverUrl, true);
        xhr.responseType = 'json';
        xhr.onerror = function(error) {
            World.updateStatusMessage("Invalid web-service response.", true);
            AR.logger.error('ProgressEvent ? '+error)
        }
        xhr.onload = function() {
            var status = xhr.status;
            AR.logger.error("is this called?");
            if (status === 200) {
                World.loadPoisFromJsonData(xhr.response);
                World.isRequestingData = false;
            } else {
                World.updateStatusMessage("Invalid web-service response.", true);
                World.isRequestingData = false;
            }
        }
        xhr.send();
    },

    /* Helper to sort places by distance. */
    sortByDistanceSorting: function sortByDistanceSortingFn(a, b) {
        return a.distanceToUser - b.distanceToUser;
    },

    /* Helper to sort places by distance, descending. */
    sortByDistanceSortingDescending: function sortByDistanceSortingDescendingFn(a, b) {
        return b.distanceToUser - a.distanceToUser;
    },

    onError: function onErrorFn(error) {
        alert(error);
    }
};


/* Forward locationChanges to custom function. */
AR.context.onLocationChanged = World.locationChanged;

/* Forward clicks in empty area to World. */
AR.context.onScreenClick = World.onScreenClick;